package	jp.co.sds_corp.hibiya.connector;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.ResourceBundle;

import jp.co.sds_corp.hibiya.adapter.AdapterFactory;
import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.hibiya.server.MonitorDaemon;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.IService;
import jp.co.sds_corp.util.daemon.core.IDaemon;
import jp.co.sds_corp.util.daemon.core.Interval;
import jp.co.sds_corp.util.daemon.manager.DaemonManager;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* 同一VM内の{@link DaemonManager} から、直接{@link IChecker}の情報を取得します。
*/
public class DirectConnector extends AbstractConnector
	implements IConnector, ICheckerListener
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	transient private DaemonManager manager;
	transient private ResourceBundle	res;

	public DirectConnector() {
		res = ResourceBundle.getBundle( "resource.server" );
		manager = DaemonManager.getInstance();
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		setConnected( true );
	}

	public void start() {
		super.start();
		Map<String,IChecker> map = getCheckerMap();
		Iterator<String> itr = map.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			IChecker chk = (IChecker )map.get( key );
			chk.addCheckerListener( this );
		}
		fireConnected( new ConnectionEvent( this ));
	}

	public void stop() {
		super.stop();
		Map<String,IChecker> map = getCheckerMap();
		Iterator<String> itr = map.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			IChecker chk = (IChecker )map.get( key );
			chk.removeCheckerListener( this );
		}
		fireDisconnected( new ConnectionEvent( this ));
		logger.debug( this, " stopped.[" + getName() + "]" );
	}

	public boolean checkConnection() {
		return true;
	}

	/**
	* {@link IChecker}を取得します。
	* @param	name	ICheckerの登録名
	*/
	public IChecker getChecker( String name ) {
		Map<String,IChecker> map = getCheckerMap();
		return (IChecker )map.get( name );
	}

	/**
	* {@link IChecker}の一覧を返します。
	* @return	ICheckerの一覧が格納されたMap
	*/
	public synchronized Map<String,IChecker> getCheckerMap() {
		LinkedHashMap<String,IChecker> ret = new LinkedHashMap<String,IChecker>();
		Map<String,IDaemon> map = manager.getDaemonMap();
		Iterator<String> itr = map.keySet().iterator();
		while( itr.hasNext() ) {
			String key = (String )itr.next();
			IDaemon dmn = (IDaemon )map.get( key );
			IService svc = dmn.getService();
			if ( svc instanceof IChecker ) {
				ret.put( svc.getName(), (IChecker )svc );
			}
		}
		return ret;
	}

	/**
	* {@link IChecker}を追加します
	* @param	chk		追加するIChecker
	*/
	public void addChecker( IChecker chk ) throws Exception {
		MonitorDaemon dmn = new MonitorDaemon();
		dmn.setInterval( new Interval( chk.getInterval() ));
		dmn.setService( chk );
		dmn.setName( chk.getName() );
		chk.addCheckerListener( dmn );
		chk.addCheckerListener( this );

		DaemonManager manager = DaemonManager.getInstance();
		manager.addDaemon( manager.getDaemonMap(), dmn );
		AdapterFactory.addChecker( chk );
		dmn.start();
		String msg = res.getString( "checker.add" );
		CheckerChangeEvent ev = new CheckerChangeEvent( chk,
			IChecker.Status.ADD, IChecker.Status.ADD, null,
			chk.getType() + " [" + chk.getName() + "] " + msg );
		chk.fireCheckerChange( ev );
	}

	/**
	* {@link IChecker}を削除します
	* @param	chk		削除するIChecker
	*/
	public void removeChecker( IChecker chk ) throws Exception {
		String msg = res.getString( "checker.delete" );
		CheckerChangeEvent ev = new CheckerChangeEvent( chk,
			chk.getCheckerStatus(), IChecker.Status.DEL, null,
			chk.getType() + " [" + chk.getName() + "] " + msg );
		chk.setCheckerStatus( IChecker.Status.DEL );
		chk.fireCheckerChange( ev );

		MonitorDaemon dmn = (MonitorDaemon )manager.removeDaemon( chk.getName() );
		chk.removeCheckerListener( dmn );
		chk.removeCheckerListener( this );
		AdapterFactory.removeChecker( chk );
	}

	public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
		fireCheckerChange( ev );
	}

}

